﻿using System;
using System.Collections.Generic;

namespace GameUtils
{
    /// <summary>
    /// A handle that points to an engine component.
    /// </summary>
    public class EngineHandle : IDisposable
    {
        LinkedList<IEngineComponent> list;
        LinkedListNode<IEngineComponent> node;
        object locker;
        bool disposed;

        internal event EventHandler Disposed;

        internal IEngineComponent Component
        {
            get { return node.Value; }
        }

        internal EngineHandle(LinkedList<IEngineComponent> list, LinkedListNode<IEngineComponent> node, object locker)
        {
            this.list = list;
            this.node = node;
            this.locker = locker;
        }

        public void Dispose()
        {
            if (!disposed)
            {
                lock (locker) list.Remove(node);
                if (Disposed != null)
                    Disposed(this, EventArgs.Empty);
                list = null;
                node = null;
                locker = null;

                disposed = true;
            }
        }
    }
}
