﻿using GameUtils.Graphics.Data;

namespace GameUtils.Graphics
{
    /// <summary>
    /// Provides additional information for text rendering.
    /// </summary>
    public class TextFormat : Resource<TextFormatData>
    {
        HorizontalAlignment horizontalAlignment;
        VerticalAlignment verticalAlignment;
        WordWrapping wordWrapping;
        bool hAlignChanged;
        bool vAlignChanged;
        bool wrappingChanged;
        readonly object locker;

        /// <summary>
        /// The horizontal alignment of the text inside the format rectangle.
        /// </summary>
        public HorizontalAlignment HorizontalAlignment
        {
            get { return horizontalAlignment; }
            set
            {
                lock (locker)
                {
                    horizontalAlignment = value;
                    hAlignChanged = true;
                }
            }
        }

        /// <summary>
        /// The vertical alignment of the text inside the format rectangle.
        /// </summary>
        public VerticalAlignment VerticalAlignment
        {
            get { return verticalAlignment; }
            set
            {
                lock (locker)
                {
                    verticalAlignment = value;
                    vAlignChanged = true;
                }
            }
        }

        /// <summary>
        /// A value indicating how words should be wrapped at the format rectangle's border.
        /// </summary>
        public WordWrapping WordWrapping
        {
            get { return wordWrapping; }
            set
            {
                lock (locker)
                {
                    wordWrapping = value;
                    wrappingChanged = true;
                }
            }
        }

        public TextFormat()
        {
            locker = new object();
        }

        protected override void ApplyChanges()
        {
            lock (locker)
            {
                if (hAlignChanged)
                {
                    Data.HorizontalAlignment = horizontalAlignment;
                    hAlignChanged = false;
                }
                if (vAlignChanged)
                {
                    Data.VerticalAlignment = verticalAlignment;
                    vAlignChanged = false;
                }
                if (wrappingChanged)
                {
                    Data.WordWrapping = wordWrapping;
                    wrappingChanged = false;
                }
            }
        }

        protected override TextFormatData CreateData(Factory factory)
        {
            lock (locker)
            {
                hAlignChanged = false;
                vAlignChanged = false;
                wrappingChanged = false;
                TextFormatData data = factory.CreateTextFormat();
                data.HorizontalAlignment = horizontalAlignment;
                data.VerticalAlignment = verticalAlignment;
                data.WordWrapping = wordWrapping;
                return data;
            }
        }
    }
}
